@@ -24,6 +24,8 @@
 #' of rows of argument profile
 #' @param day Number of considered days in estimation
 #' @param verbose Logical; To show more information
+#' @param nt Integer; Number of threads wich must be lower than max available.
+#' See \code{\link{check_nt}}. Only when fortran = TRUE
 #' @useDynLib  vein, .registration = TRUE
 #' @return If the user applies a top-down approach, the resulting units will be
 #' according its own data. For instance, if the vehicles are veh/day, the units
@@ -32,6 +34,7 @@
 #' matrix.
 #' @export
 #' @importFrom sf st_set_geometry
+#' @importFrom dotCall64 .C64 numeric_dc
 #' @examples \dontrun{
 #' # Do not run
 #' data(net)
@@ -154,7 +157,8 @@ emis <- function (veh,
                   fortran = FALSE,
                   hour = nrow(profile),
                   day = ncol(profile),
-                  verbose = FALSE) {
+                  verbose = FALSE,
+                  nt = ifelse(check_nt()==1, 1, check_nt()/2)) {
   # Check units
   if(class(lkm) != "units"){
     stop("lkm neeeds to has class 'units' in 'km'. Please, check package 'units'")
@@ -199,16 +203,48 @@ emis <- function (veh,
         if(length(lkm) != nrow(veh)) stop("length of `lkm` must be equal to number of rows of `veh`")
         if(length(ef) != ncol(veh)) stop("length of `ef` and number of cols of `veh` must be equal")
         # emis(i, j) = veh(i,j) * lkm(i) * ef(j)
+        if(!missing(nt)) {
+          if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                    " threads and nt must be lower")
 
+          if(verbose) message("Calling emis2dfpar.f95")
+          a <- dotCall64::.C64(
+            .NAME = "emis2dfpar",
+            SIGNATURE = c("integer", "integer",
+                          "double", "double", "double",
+                          "integer", "double"),
+            nrowv = nrowv,
+            ncolv = ncolv,
+            veh = veh,
+            lkm = lkm,
+            ef = ef,
+            nt = as.integer(nt),
+            emis = dotCall64::numeric_dc(nrowv*ncolv),
+            INTENT = c("r", "r",
+                       "r", "r","r",
+                       "r","w"),
+            PACKAGE = "vein",
+            VERBOSE = 1)$emis
+
+        } else {
           if(verbose) message("Calling emis2df.f95")
-          a <-   .Fortran("emis2df",
-                          nrowv = nrowv,
-                          ncolv = ncolv,
-                          veh = veh,
-                          lkm = lkm,
-                          ef = ef,
-                          emis = numeric(nrowv*ncolv))$emis
-
+          a <- dotCall64::.C64(
+            .NAME = "emis2df",
+            SIGNATURE = c("integer", "integer",
+                          "double", "double", "double",
+                          "double"),
+            nrowv = nrowv,
+            ncolv = ncolv,
+            veh = veh,
+            lkm = lkm,
+            ef = ef,
+            emis = dotCall64::numeric_dc(nrowv*ncolv),
+            INTENT = c("r", "r",
+                       "r", "r","r",
+                       "w"),
+            PACKAGE = "vein",
+            VERBOSE = 1)$emis
+        }
 
         e <- matrix(a, nrow = nrowv, ncol =  ncolv)
         return(Emissions(e))
@@ -329,18 +365,53 @@ emis <- function (veh,
           if(length(ef) != ncol(veh)) stop("length of `ef` and number of cols of `veh` must be equal")
           # emis(i, j, k) = veh(i, j) * lkm(i) * ef(j)*pro(k)
 
+          if(!missing(nt)) {
+            if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                      " threads and nt must be lower")
+
+            if(verbose) message("Calling emis3dfpar.f95")
+            a <-   dotCall64::.C64(
+              .NAME = "emis3dfpar",
+              SIGNATURE = c("integer", "integer","integer",
+                            "double", "double", "double","double",
+                            "integer", "double"),
+              nrowv = nrowv,
+              ncolv = ncolv,
+              prok = prok,
+              veh = veh,
+              lkm = lkm,
+              ef = ef,
+              pro = profile,
+              nt = as.integer(nt),
+              emis = dotCall64::numeric_dc(nrowv*ncolv*prok),
+              INTENT = c("r", "r", "r",
+                         "r", "r", "r", "r",
+                         "r","w"),
+              PACKAGE = "vein",
+              VERBOSE = 1)$emis
+
+          } else {
 
             if(verbose) message("Calling emis3df.f95")
-            a <-   .Fortran("emis3df",
-                            nrowv = nrowv,
-                            ncolv = ncolv,
-                            prok = prok,
-                            veh = veh,
-                            lkm = lkm,
-                            ef = ef,
-                            pro = profile,
-                            emis = numeric(nrowv*ncolv*prok))$emis
-
+            a <-   dotCall64::.C64(
+              .NAME = "emis3df",
+              SIGNATURE = c("integer", "integer","integer",
+                            "double", "double", "double","double",
+                            "double"),
+              nrowv = nrowv,
+              ncolv = ncolv,
+              prok = prok,
+              veh = veh,
+              lkm = lkm,
+              ef = ef,
+              pro = profile,
+              emis = numeric(nrowv*ncolv*prok),
+              INTENT = c("r", "r", "r",
+                         "r", "r", "r", "r",
+                         "w"),
+              PACKAGE = "vein",
+              VERBOSE = 1)$emis
+          }
           # fortran
           # do concurrent(i= 1:nrowv, j = 1:ncolv, k = 1:prok)
           # emis(i, j,k) = veh(i,j) * lkm(i) * ef(j)*pro(k)
@@ -369,19 +440,55 @@ emis <- function (veh,
           if(length(ef) != ncol(veh)) stop("length of `ef` and number of cols of `veh` must be equal")
           # emis(i, j, k, l) = veh(i,j) * lkm(i) * ef(j)*pro(k,l)
 
+          if(!missing(nt)) {
 
+            if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                      " threads and nt must be lower")
+
+            if(verbose) message("Calling emis4dfpar.f95")
+            a <-   dotCall64::.C64(
+              .NAME = "emis4dfpar",
+              SIGNATURE = c("integer", "integer", "integer", "integer",
+                            "double", "double", "double", "double",
+                            "integer", "double"),
+              nrowv = nrowv,
+              ncolv = ncolv,
+              proh = proh,
+              prod = prod,
+              veh = veh,
+              lkm = lkm,
+              ef = ef,
+              pro = profile,
+              nt = as.integer(nt),
+              emis = dotCall64::numeric_dc(nrowv*ncolv*proh*prod),
+              INTENT = c("r", "r", "r", "r",
+                         "r", "r", "r", "r",
+                         "r","w"),
+              PACKAGE = "vein",
+              VERBOSE = 1)$emis
+          } else {
             if(verbose) message("Calling emis4df.f95")
+            a <-   dotCall64::.C64(
+              .NAME = "emis4df",
+              SIGNATURE = c("integer", "integer", "integer", "integer",
+                            "double", "double", "double", "double",
+                            "double"),
+              nrowv = nrowv,
+              ncolv = ncolv,
+              proh = proh,
+              prod = prod,
+              veh = veh,
+              lkm = lkm,
+              ef = ef,
+              pro = profile,
+              emis = dotCall64::numeric_dc(nrowv*ncolv*proh*prod),
+              INTENT = c("r", "r", "r", "r",
+                         "r", "r", "r", "r",
+                         "w"),
+              PACKAGE = "vein",
+              VERBOSE = 1)$emis
 
-            a <-   .Fortran("emis4df",
-                            nrowv = nrowv,
-                            ncolv = ncolv,
-                            proh = proh,
-                            prod = prod,
-                            veh = veh,
-                            lkm = lkm,
-                            ef = ef,
-                            pro = profile,
-                            emis = numeric(nrowv*ncolv*proh*prod))$emis
+          }
           e <- array(a, dim = c(nrowv, ncolv,proh, prod))
           return(EmissionsArray(e))
 
